/*
 *	PostgreSQL type definitions for rational numbers
 *
 *	$Id: rational-glue.c 1565 2010-06-18 20:22:51Z stephane $
 */

#include <postgres.h>
#include <fmgr.h>

#include "rational.h"

#ifdef PG_MODULE_MAGIC
PG_MODULE_MAGIC;
#endif

/*
 *	RATIONAL reader.
 */

PG_FUNCTION_INFO_V1(rational_in);
Datum
rational_in(PG_FUNCTION_ARGS)
{
    char           *str = PG_GETARG_CSTRING(0);
    rational       *result, val;
    result = MALLOC(sizeof(rational));
    val = text_to_rational(str);
    if (!VALID(val)) {
        ereport(ERROR, (errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
                        errmsg("rational_in: invalid RATIONAL \"%s\"", str)));
    }
    memcpy(result, &val, sizeof(rational));
    PG_RETURN_POINTER(result);
}

/*
 *	RATIONAL output function.
 */

PG_FUNCTION_INFO_V1(rational_out);
Datum
rational_out(PG_FUNCTION_ARGS)
{
    char           *result, *val;
    rational       *num = (rational *) PG_GETARG_POINTER(0);
    if (num == NULL)
        PG_RETURN_NULL();
    result = (char *) MALLOC(MAX_RATIONAL_LEN);
    val = rational_to_text(*num);
    strcpy(result, val);
    PG_RETURN_CSTRING(result);
}

/*
 *	Boolean tests for magnitude.
 */

PG_FUNCTION_INFO_V1(rational_lt);
Datum
rational_lt(PG_FUNCTION_ARGS)
{
    bool            result;
    result =
        is_lt(*((rational *) PG_GETARG_POINTER(0)),
              *((rational *) PG_GETARG_POINTER(0)));
    PG_RETURN_BOOL(result);
};

PG_FUNCTION_INFO_V1(rational_le);
Datum
rational_le(PG_FUNCTION_ARGS)
{
    bool            result;
    result =
        is_le(*((rational *) PG_GETARG_POINTER(0)),
              *((rational *) PG_GETARG_POINTER(0)));
    PG_RETURN_BOOL(result);
};

PG_FUNCTION_INFO_V1(rational_eq);
Datum
rational_eq(PG_FUNCTION_ARGS)
{
    bool            result;
    result =
        are_equal(*((rational *) PG_GETARG_POINTER(0)),
                  *((rational *) PG_GETARG_POINTER(0)));
    PG_RETURN_BOOL(result);
};

PG_FUNCTION_INFO_V1(rational_ge);
Datum
rational_ge(PG_FUNCTION_ARGS)
{
    bool            result;
    result =
        is_ge(*((rational *) PG_GETARG_POINTER(0)),
              *((rational *) PG_GETARG_POINTER(0)));
    PG_RETURN_BOOL(result);
};

PG_FUNCTION_INFO_V1(rational_gt);
Datum
rational_gt(PG_FUNCTION_ARGS)
{
    bool            result;
    result =
        is_gt(*((rational *) PG_GETARG_POINTER(0)),
              *((rational *) PG_GETARG_POINTER(0)));
    PG_RETURN_BOOL(result);
};

PG_FUNCTION_INFO_V1(rational_ne);
Datum
rational_ne(PG_FUNCTION_ARGS)
{
    bool            result;
    result =
        !are_equal(*((rational *) PG_GETARG_POINTER(0)),
                   *((rational *) PG_GETARG_POINTER(0)));
    PG_RETURN_BOOL(result);
};

/*
 *	Comparison function for sorting:
 */

int4
rational_cmp(rational * l, rational * r)
{
    return (cmp(*l, *r));
}
