#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>

#include "rational.h"

bool
are_equal(rational left, rational right)
{
    return ((left.numerator == right.numerator) &&
            (left.denominator == right.denominator));
}

bool
is_lt(rational left, rational right)
{
    return (float_of(left) < float_of(right));
}

bool
is_le(rational left, rational right)
{
    return (float_of(left) <= float_of(right));
}

bool
is_gt(rational left, rational right)
{
    return (float_of(left) > float_of(right));
}

bool
is_ge(rational left, rational right)
{
    return (float_of(left) >= float_of(right));
}

int
cmp(rational left, rational right)
{
    if (are_equal(left, right)) {
        return 0;
    } else if (float_of(left) < float_of(right)) {
        return -1;
    } else {
        return 1;
    }
}

double
float_of(const rational r)
{
    return ((double) r.numerator / (double) r.denominator);
}

void
_canonicalize(rational * r)
{
    long            denom = gcd(r->numerator, r->denominator);
    if (denom != 1) {
        r->numerator = r->numerator / denom;
        r->denominator = r->denominator / denom;
    }
}

/* Input-Output functions */
rational
text_to_rational(const char *textr)
{
    rational        r;
    char           *slash;
    unsigned int    i;
    INIT(r);
    slash = index(textr, '/');
    if (slash == NULL) {
        r.denominator = 1;
        for (i = 0; i < strlen(textr); i++) {
            if (!isdigit(textr[i])) {
                INIT(r);
                return (r);
            }
        }
        r.numerator = atol(textr);
    } else {
        r.denominator = atol(slash + 1);
        for (i = 0; i < strlen(slash + 1); i++) {
            if (!isdigit(textr[i])) {
                INIT(r);
                return r;
            }
        }
        *slash = '\0';
        for (i = 0; i < strlen(textr); i++) {
            if (!isdigit(textr[i])) {
                INIT(r);
                return r;
            }
        }
        r.numerator = atol(textr);
    }
    _canonicalize(&r);
    return r;
}

char           *
rational_to_text(const rational r)
{
    char           *result;
    if (r.denominator == 1) {
        result = malloc(MAX_RATIONAL_LEN);
        snprintf(result, MAX_RATIONAL_LEN, "%li", r.numerator);
    } else {
        result = malloc(MAX_RATIONAL_LEN);
        snprintf(result, MAX_RATIONAL_LEN, "%li/%li", r.numerator, r.denominator);
    }
    return result;
}
